﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//      Для работы с RF-1996 
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
namespace NRF1996
{
    class CRF1996
    {
        public const string ZpDllName = "rf1996.dll";

        // Коды ошибок библиотеки
        public enum EErrorCode : int // FixME, проблемы с Window x64 
        {
            rf_SUCCESS = 0,                     // Операция выполнена успешно
            rf_INVALID_PARAM = -1,              // Неверный параметр
            rf_OPEN_NOT_EXIST = -2,             // Порт не существует
            rf_OPEN_ACCESS = -3,                // Порт занят другой программой
            rf_NOT_USB = -4,                    // Это не USB порт
            rf_OPEN_PORT = -5,                  // Неизвестная ошибка открытия порта
            rf_UNKNOWN_DEVICE = -6,             // Неизвестное устройство
            rf_DEVICE_NOT_FOUND = -7,           // Устройство не найдено
            rf_ALREADY_OPEN = -8,               // Порт уже окрыт (при проблемах с синхронизацией)

            rf_EEPROM_ERROR = -9,               // Не читается ключ шифрации
            rf_DATA_ERROR = -10,                // Не прочитать данные - повторить чтение
            rf_DEVICE_NOT_CONNECT = -11,        // Нет соединения с устройством
            rf_NO_CARD = -12,                   // Нет карты на адаптере
            rf_DLL_NOT_INIT = -14,              // Библиотека не инициализирована
            rf_NO_ANSWER_FOR_CMD = -15,         // Нет ответа на команду
            rf_DISPLAY_MODE_ERROR = -16,        // Попытка установки состояния светодиодов в автоматическом режиме
            rf_UNKNOWN_ERROR = -17,             // Неизвестная ошибка
            rf_FAIL_GET_LIST_COM_PORT = -18,    // Не удалось получить список COM-портов
            rf_PORT_NOT_OPEN = -19,             // Порт не открыт

            rf_ERROR_LICENSE_HARD = -20,        // Аппаратные ошибки лицензии
            rf_LICENSE_FINISHED = -21,          // Лицензия закончилась
            rf_LICENSE_NOT_VALID = -22,         // Лицензия недействительна

            rf_CARD_NOT_FOUND = -30,            // Карта не найдена
            rf_BAD_PASSWORD = -31,              // Ошибка пароля
            rf_ERROR_SAVE_CARD = -32,           // Ошибка записи карты

            rf_ERROR_INIT_DLL = -40,            // Ошибка инициализации библиотеки

            els_no_info = 0,                    // Нет информации
            els_ok = 0xAA,                      // Лицензия действительна
            els_none = 0xFE,                    // Лицензия не действительная
            els_conclude = 0xFD,                // Лицензия закончилась
            els_hard_error = 0xFC,              // Аппаратные ошибки
        };


        // Состояние лицензии на адаптере RF-1996
        public enum ELicenseStatus : uint // FixME, проблемы с Window x64 
        {
            els_no_info     = 0,    // Нет информации
            els_ok          = 0xAA, // Лицензия действительна
            els_none        = 0xFE, // Лицензия не действительная
            els_conclude    = 0xFD, // Лицензия закончилась
            els_hard_error  = 0xFC, // Аппаратные ошибки
        };
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Информация об адаптере RF-1996
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////
        [StructLayout(LayoutKind.Sequential, Pack = 4, CharSet = CharSet.Unicode)]

        public struct SDeviceInfoW
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
            public string   Port;                   // Com-порт устройства

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
            public string   DeviceName;             // Наименование устройства

            public UInt32   SerialNumber;           // Серийный номер устройства
            public UInt32   ModelNumber;            // Номер модели
            public UInt32   FirmwareVersion;        // Версия Firmware
            public ELicenseStatus LicenseStatus;    // Состояние лицензии на адаптере
            public UInt32   LicenseYear;            // Год
            public UInt32   LicenseMonth;           // Месяц
            public UInt32   LicenseDay;             // День выпуска лицензии
            public UInt32   CountCard;              // Кол-во карт, доступных для стирания-записи по лицензии

            /////////////////////////////////////////////////////////////////////////////////////////////////////////////
            //
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////
            public SDeviceInfoW( int Flag )     // FixME, Flag not use
            {
                Port = "";                      // Com-порт устройства
                DeviceName = "";                // Наименование устройства 

                SerialNumber = 0;               // Серийный номер устройства
                ModelNumber  = 0;               // Номер модели
                FirmwareVersion = 0;            // Версия Firmware
                LicenseStatus = ELicenseStatus.els_no_info;  // Состояние лицензии на адаптере
                LicenseYear  = 0;               // Год
                LicenseMonth = 0;               // Месяц
                LicenseDay   = 0;               // День выпуска лицензии
                CountCard    = 0;               // Кол-во карт, доступных для стирания-записи по лицензии
            }
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////
            //
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////
        };

        public enum ECardType : uint        // FixME, проблемы с Window x64 
        {
            ect_none         = 0,			// Нет карты на адаптере
            ect_clean_temic  = 0x10,		// Чистый Темик (карта без пароля)
            ect_temic        = 0x20,		// Записанный Темик (пароль подошел)
            ect_bad_password = 0x30,		// "Грязный Темик" (пароль не подошел)
            esi_em_marine    = 0x40,		// Em-Marine (21 февраля 2012)
            // Для внутреннего применения
            ect_EEPROM_error = 0x80,		// Не читается ключ
            ect_data_error   = 0x81,		// Не прочитать данные - повторить чтение

            ect_unknow       = 0xF0,		// Неизвестный код
        };

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Информация о карте на адаптере RF-1996
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////
        
        // Флаги действительности значения полей структуры SCardInfo
        // Примечание. Флаг FCI_LOCK_INFO валидные поля: CountBorrow, NumLastLocks, BatteryCondition
        public const UInt32 FCI_TEMIC       = 1;
        public const UInt32 FCI_EM_MARINE   = 2;
        public const UInt32 FCI_LIMIT_LOCKS = 4;
        public const UInt32 FCI_LOCK_INFO   = 8;

        [StructLayout(LayoutKind.Sequential, Pack = 4)] // CharSet отдыхает
        public struct SCardInfo
        {
            public UInt32     Flags;            // Флаги действительности значений полей структур (см. ниже)
            public ECardType  CardType;         // Тип карты

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public byte[]     TemicCode;        // Код Temic 

            [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
            public byte[]     EmMarineCode;     // Код Em-Marine
            
            public UInt32     LimitLocks;       // Максимальное кол-во шкафчиков, которые можно занять данной картой

            public UInt32     CountBorrow;      // Кол-во занятых шкафчиков
            public UInt32     NumLastLocks;     // Номер последнего занятого шкафчика
            public UInt32     BatteryCondition; // Состояние батареи: TRUE - норма, FALSE - требуется замена

            // Для оптимизации кода
            public void GetCardType(ref string str)
            {
                switch (CardType)
                {
                    case ECardType.ect_clean_temic: str = "Чистый Темик"; break;
                    case ECardType.ect_temic: str = "Темик с паролем"; break;
                    case ECardType.ect_bad_password: str = "Неверный пароль"; break;
                    case ECardType.esi_em_marine: str = "Em-Marine"; break;
                    default: str = "Неизвестное значение"; break;
                }
            }

            // Получить тип карты

            // Вывести в читабельном виде код Temic
            public void GetTemicCode(ref string str)
            {
                str = "";
                if ((Flags & NRF1996.CRF1996.FCI_TEMIC) != 0)
                {
                    for (int i = 0; i < 8; i++)
                    {
                        str += String.Format("{0:x2} ", TemicCode[i]);
                    }
                }
            }

            // Вывести в читабельном виде код Em-marine
            public void GetEmMarineCode(ref string str)
            {
                str = "";
                if ( (Flags & NRF1996.CRF1996.FCI_EM_MARINE ) != 0 )
                {
                    int SquareBrackets = (EmMarineCode[4] << 8) + EmMarineCode[3];
                    int Decimal = (EmMarineCode[1] << 8) + EmMarineCode[0];

                    str = String.Format("[{0:X4}] {1:d}, {2:d}", SquareBrackets, EmMarineCode[2], Decimal);
                }
            }

            // Вывести лимит шкафчиков
            public void GetLimitBoxes(ref string str)
            {
                str = "";
                if ( (Flags & NRF1996.CRF1996.FCI_LIMIT_LOCKS) != 0)
                {
                    str = LimitLocks.ToString();
                }
            }

            // Вывести кол-во занятых шкафчиков
            public void GetCountBorrow(ref string str)
            {
                str = "";
                if ( (Flags & NRF1996.CRF1996.FCI_LOCK_INFO) != 0)
                {
                    str = CountBorrow.ToString();
                }
            }

            // Вывести последний занятый
            public void GetNumLastLocks(ref string str)
            {
                str = "";
                if ((Flags & NRF1996.CRF1996.FCI_LOCK_INFO) != 0)
                {
                    str = NumLastLocks.ToString();
                }
            }

            // Вывести состояние батареи
            public void GetBatteryCondition(ref string str)
            {
                str = "";
                if ((Flags & NRF1996.CRF1996.FCI_LOCK_INFO) != 0)
                {
                    if( 0 == BatteryCondition )
                        str = "Разряжена";
                    else
                        str = "Норма";
                }
            }

        };
        //Функции библиотеки

        // Инициализация библиотеки
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "Init")]
        public static extern Int32 Init( );

        // Функция возвращает статус считывателя - есть соединение с RF1996 или нет 
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsDeviceOpen")]
        public static extern Int32 IsDeviceOpen( );

        // Функция открывает устройство
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SearchDevice")]
        public static extern Int32 SearchDevice( );

        // Функция открывает устройство
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "OpenDeviceW")]
        public static extern Int32 OpenDeviceW([MarshalAs(UnmanagedType.LPWStr)] String s );

        // Функция закрывает устройство
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "CloseDevice")]
        public static extern Int32 CloseDevice( );

        // Получить информацию об RF-1996
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "GetDeviceInfoW")]
        public static extern Int32 GetDeviceInfoW( ref SDeviceInfoW DevInfo );

        // Получить информацию о карте на адаптере RF-1996
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReadCard")]
        public static extern Int32 ReadCard(ref SCardInfo CardInfo);

        // Режим работы индикации адаптера RF-1996
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetIndicationMode")]
        public static extern Int32 SetIndicationMode( Int32 mode );

        // Включить-выключить светодиоды
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetDisplay")]
        public static extern Int32 SetDisplay(Int32 fRed, Int32 fGreen );

        // Включить-выключить красный светодиод
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetDisplayRed")]
        public static extern Int32 SetDisplayRed( Int32 fRed );

        // Включить-выключить зеленый светодиод
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetDisplayGreen")]
        public static extern Int32 SetDisplayGreen(Int32 fGreen);

        // Стереть карту на адаптере
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "ClearCard")]
        public static extern Int32 ClearCard( );

        // Установить асинхронный режим работы SendMessage
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetModeSelectMsg")]
        public static extern Int32 SetModeSelectMsg( IntPtr hWnd, Int32 Msg, Int32 Delay);

        // Установить асинхронный режим работы SetEvent
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetModeSelectEvent")]
        public static extern Int32 SetModeSelectEvent(UInt32 hEvent, Int32 Delay);

        // Инициализировать карту
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "InitCard")]
        public static extern Int32 InitCard( byte[] EmMarineCode );

        // Выписать карту
        [DllImport(ZpDllName, CallingConvention = CallingConvention.StdCall, EntryPoint = "WriteCard")]
        public static extern Int32 WriteCard(Int32 LimitLocks);

    };
};
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////